@@ -1,21 +1,7 @@
 
-####### This, essentially, should be added as a new function: chainsPlot2, or
-####### change chainsPlot -> chainsSummary(), and make this below be chainsPlot()
-####### 
-#######par(mfrow = c(7,3), mai = c(0.3,0.3,0.2,0.2))
-####### 
-#######for(i in 1:21){
-#######        
-#######        plot(mcmcComb[[1]]$samples[1,i,], type = 'l', main = dimnames(mcmcComb[[1]]$samples)[[2]][i], ylim = c(min(mcmcComb[[1]]$samples[1:3,i,]), max(mcmcComb[[1]]$samples[1:3,i,])), xlab ='', ylab ='')
-#######        lines(mcmcComb[[1]]$samples[2,i,], col = 'red', lty = 2)
-#######        lines(mcmcComb[[1]]$samples[3,i,], col = 'green', lty = 3)
-#######}
-####### 
-
-
 #' Plot MCMC traceplots and density plots
 #'
-#' @param samples Array of MCMC samples
+#' @param samples Array of MCMC samples, or a list of samples from multiple chains in which case the first chain is used
 #' @param var Parameter names to plot
 #' @param ind Chain indices to plot
 #' @param burnin Nunber of initial MCMC samples to discard
@@ -42,6 +28,7 @@ samplesPlot <- function(samples, var=colnames(samples), ind=NULL, burnin=NULL, s
     par.save <- par(no.readonly = TRUE)
     par(mfrow=c(1,traceplot+densityplot), cex=0.7, cex.main=1.5, cex.axis=0.9, lab=c(3,3,7), mgp=c(0,0.4,0), mar=c(1.6,1.6,2,0.6), oma=c(0,0,0,0), tcl=-0.3, bty='l')
     ## process samples
+    if(is.list(samples)) samples <- samples[[1]]    ## if samplesList was provided, then take first chain
     var <- gsub('\\[', '\\\\\\[', gsub('\\]', '\\\\\\]', var))   ## add \\ before any '[' or ']' appearing in var
     var <- unlist(lapply(var, function(n) grep(paste0('^', n,'(\\[.+\\])?$'), colnames(samples), value=TRUE)))  ## expanded any indexing
     samples <- samples[, var, drop=FALSE]
@@ -71,15 +58,15 @@ samplesPlot <- function(samples, var=colnames(samples), ind=NULL, burnin=NULL, s
         if(legend & !is.null(dimnames(samples)) & is.character(dimnames(samples)[[2]]))
             legend(legend=dimnames(samples)[[2]], fill=rainbow(nparam, alpha=0.5), bty='n', x=legend.location)
     }  ## finish densityplot
-    if(!is.null(file)) dev.off()
     invisible(par(par.save))
+    if(!is.null(file)) dev.off()
 }
  
 
 
 #' Compare summary statistics from multiple MCMC chains
 #'
-#' Parameter plots from each chain show median and 95% credible intervals
+#' Plots median and 95% credible intervals for each paramter and chain.
 #'
 #' @param samplesList List of arrays of MCMC samples from different chains
 #' @param var Parameter names to plot
@@ -101,10 +88,10 @@ samplesPlot <- function(samples, var=colnames(samples), ind=NULL, burnin=NULL, s
 #' samples2 <- cbind(rnorm(1000, 2), rgamma(1000, 2), rpois(1000, 2))
 #' colnames(samples2) <- c('alpha', 'beta', 'gamma')
 #' samplesList <- list(chain1 = samples1, chain2 = samples2)
-#' chainsPlot(samplesList, nrow = 1, jitter = .3, buffer.left = .5, buffer.right = .5)
+#' chainsSummary(samplesList, nrow = 1, jitter = .3, buffer.left = .5, buffer.right = .5)
 #'
 #' @export
-chainsPlot <- function(samplesList, var=NULL, nrows=NULL, scale=FALSE, width=7, height=NULL, legend=!is.null(names(samplesList)), legend.location='topright', jitter=1, buffer.right=0, buffer.left=0, cex=1, file=NULL) {
+chainsSummary <- function(samplesList, var=NULL, nrows=NULL, scale=FALSE, width=7, height=NULL, legend=!is.null(names(samplesList)), legend.location='topright', jitter=1, buffer.right=0, buffer.left=0, cex=1, file=NULL) {
     if(!(class(samplesList) %in% c('list', 'mcmc.list'))) samplesList <- list(samplesList)
     if(!is.null(var)) samplesList <- lapply(samplesList, function(samples) {
         var <- gsub('\\[', '\\\\\\[', gsub('\\]', '\\\\\\]', var))   ## add \\ before any '[' or ']' appearing in var
@@ -161,14 +148,99 @@ chainsPlot <- function(samplesList, var=NULL, nrows=NULL, scale=FALSE, width=7,
             points(x=xsJittered[ps], y=summary[iChain,'mean',ps], pch=16, col=cols[iChain])
             segments(x0=xsJittered[ps], y0=summary[iChain,'low',ps], y1=summary[iChain,'upp',ps], lwd=1, col=cols[iChain])
         }
-        if(legend) legend(legend.location, legend=names(samplesList), pch=16, col=cols, cex=cex)
+        if(legend & !is.null(names(samplesList))) legend(legend.location, legend=names(samplesList), pch=16, col=cols, cex=cex)
     }
-    if(!is.null(file)) dev.off()
     invisible(par(par.save))
+    if(!is.null(file)) dev.off()
 }
 
 
 
+
+
+#' Compare trace plots from multiple MCMC chains
+#'
+#' Overlays trace plots from each MCMC chain, for each parameter
+#'
+#' @param samplesList List of arrays of MCMC samples from different chains
+#' @param var Parameter names to plot
+#' @param scale Logical, whether to normalize each posterior chain
+#' @param legend Logical, whether to include a legend of chain names
+#' @param legend.location Legend location
+#' @param cex Expansion coefficient for text
+#' @param densityplot Logical, whether to generate posterior density plots instead of trace plots
+#' @param file Filename for saving figure to a file
+#'
+#' @examples
+#' samples1 <- cbind(rnorm(1000, 1), rgamma(1000, 1), rpois(1000, 1))
+#' colnames(samples1) <- c('alpha', 'beta', 'gamma')
+#' samples2 <- cbind(rnorm(1000, 2), rgamma(1000, 2), rpois(1000, 2))
+#' colnames(samples2) <- c('alpha', 'beta', 'gamma')
+#' samplesList <- list(chain1 = samples1, chain2 = samples2)
+#' chainsPlot(samplesList)
+#'
+#' @export
+chainsPlot <- function(samplesList, var=NULL, scale=FALSE, legend=!is.null(names(samplesList)), legend.location='topright', cex=1, densityplot=FALSE, file=NULL) {
+    if(!(class(samplesList) %in% c('list', 'mcmc.list'))) samplesList <- list(samplesList)
+    if(!is.null(var)) samplesList <- lapply(samplesList, function(samples) {
+        var <- gsub('\\[', '\\\\\\[', gsub('\\]', '\\\\\\]', var))   ## add \\ before any '[' or ']' appearing in var
+        theseVar <- unlist(lapply(var, function(n) grep(paste0('^', n,'(\\[.+\\])?$'), colnames(samples), value=TRUE)))  ## expanded any indexing
+        ret <- samples[, theseVar, drop=FALSE]
+        if(dim(ret)[2] == 0) stop('variable names misspelled', call. = FALSE)
+        ret
+    })
+    chainParamNamesList <- lapply(samplesList, function(s) colnames(s))
+    nChains <- length(samplesList)
+    paramNamesAll <- unique(unlist(lapply(samplesList, function(s) colnames(s))))
+    nParamsAll <- length(paramNamesAll)
+    nrows <- ceiling(nParamsAll / 3)
+    ncols <- min(nParamsAll, 3)
+    height <- if(nrows==1) 3 else if(nrows==2) 4 else if(nrows==3) 5 else if(nrows==4) 6 else 6.5
+    width <- 7
+    if(!is.null(file)) pdf(file, width=width, height=height) else
+    ## orig: if(inherits(try(knitr::opts_chunk$get('dev'), silent=TRUE), 'try-error') || is.null(knitr::opts_chunk$get('dev')))   ## if called from Rmarkdown/knitr
+    if(inherits(try(eval(parse(text='knitr::opts_chunk$get(\'dev\')')[[1]]), silent=TRUE), 'try-error') || is.null(eval(parse(text='knitr::opts_chunk$get(\'dev\')')[[1]])))
+        dev.new(width=width, height=height)
+    par.save <- par(no.readonly = TRUE)
+    oma1 <- if(nrows==1) 0.4 else if(nrows==2) 0.4 else 0.5
+    mai1 <- if(!densityplot) 0.1 else 0.2
+    par(mfrow=c(nrows,ncols), mai=c(mai1,0.2,0.2,0.2), oma=c(oma1,.3,0,0), mgp=c(2,0.3,0))
+    for(iParam in 1:nParamsAll) {
+        thisParamName <- paramNamesAll[iParam]
+        if(!densityplot) {  ## traceplots
+            cols <- rainbow(nChains)
+            xlim <- c(1, max(unlist(lapply(samplesList, function(s) if(thisParamName %in% colnames(s)) dim(s)[1] else NULL))))
+            ylim <- range(unlist(lapply(samplesList, function(s) if(thisParamName %in% colnames(s)) s[,thisParamName] else NULL)))
+            plot(-1, -1, xlim=xlim, ylim=ylim, xlab='', ylab='', main=thisParamName, cex.main=cex, cex.axis=0.8*cex, tcl=-0.2, xaxt='n')
+            if(iParam==1 & legend & !is.null(names(samplesList))) legend(legend.location, legend=names(samplesList), lty=1, col=cols, cex=cex)
+            for(iChain in 1:nChains) {
+                if(!(thisParamName %in% colnames(samplesList[[iChain]]))) next
+                ys <- samplesList[[iChain]][,thisParamName]
+                lines(seq_along(ys), ys, col=cols[iChain]) }
+        }  ## end trace plots
+        if(densityplot) {  ## density plots
+            xMin <- xMax <- yMax <- NULL
+            for(iChain in 1:nChains) {
+                if(!(thisParamName %in% colnames(samplesList[[iChain]]))) next
+                d <- density(samplesList[[iChain]][,thisParamName])
+                xMin <- min(xMin,d$x); xMax <- max(xMax,d$x); yMax <- max(yMax, d$y) }
+            plot(-1, -1, xlim=c(xMin,xMax), ylim=c(0,yMax), type='n', main=thisParamName, xlab='', ylab='', tcl=-0.2, yaxt='n', cex.main=cex, cex.axis=0.8*cex)
+            if(iParam==1 & legend & !is.null(names(samplesList))) legend(legend.location, legend=names(samplesList), fill=rainbow(nChains, alpha=0.5), bty='n', cex=cex)
+            for(iChain in 1:nChains) {
+                if(!(thisParamName %in% colnames(samplesList[[iChain]]))) next
+                ys <- samplesList[[iChain]][,thisParamName]
+                polygon(density(ys), col=rainbow(nChains, alpha=0.2)[iChain], border=rainbow(nChains, alpha=0.2)[iChain])
+            }
+        }  ## end density plots
+    }
+    invisible(par(par.save))
+    if(!is.null(file)) dev.off()
+}
+
+
+
+
+
 ##### IN MCMC_utils.R
 #####
 #####samplesSummary <- function(samples) {
